package ejb30.issue;

import java.io.*;
import java.util.*;
import javax.ejb.*;
import javax.persistence.*;
import java.lang.reflect.*;

import com.sun.ejte.ccl.reporter.SimpleReporterAdapter;

public class TestClient {
    
    private static @EJB TestSessionRemote session;
    private static boolean dbgFlag = false;
    
    public TestClient() {
    }
      
       
    public boolean initializeData() {
        
        boolean pass=false;

        try {
            pass=session.initializeData();
        } catch(Throwable e){
            e.printStackTrace();
        }
        return pass;
        
    }

    public boolean testCurrentTime() {
        
        boolean pass=false;

        try {
            pass = session.testCurrentTime();
        } catch(Throwable e){
            e.printStackTrace();
        }

        return pass;
        
    }

    public boolean testCurrentTimestamp() {
        
        boolean pass=false;

        try {
            pass = session.testCurrentTimestamp();
        } catch(Throwable e){
            e.printStackTrace();
        }

        return pass;
        
    }

    public boolean testCurrentDate() {
        
        boolean pass=false;

        try {
            pass = session.testCurrentDate();
        } catch(Throwable e){
            e.printStackTrace();
        }

        return pass;
        
    }

    public boolean executeTest(String testname, Object test) {
	Method m=null;
	boolean ret=false;  
	
	try {
	  m = ejb30.issue.TestClient.class.getMethod(testname, new Class[] {});
	  Object[] arg = {};
	  ret = (Boolean) m.invoke(test, (Object[])null);     
	} catch (Exception ex){
	  ex.printStackTrace();
	}

	return ret;

    } 

    public static void dbg(String msg, boolean flag) {
	if (flag) {
            System.out.println(msg);
        }

    }


    public static void main(String[] args) {      
	SimpleReporterAdapter stat = new SimpleReporterAdapter("appserver-sqe");   
	stat.addDescription("EJB3 Issue 1886 Tests");	   
        String testSuiteID = "test";
        String flag = "dbg";

       	String testlist[]={"initializeData",
                           "testCurrentTime",
                           "testCurrentTimestamp",
			   "testCurrentDate"};


        if (args.length == 1) {
           testSuiteID=args[0];
        } 
        if (args.length == 2) {
           testSuiteID=args[0];
           flag=args[1];
           if (flag.equals(new String("true"))) {
             dbgFlag = true;
	   }
        } 

        TestClient tc = new TestClient();

	for (int i=0; i < testlist.length; i++){
	  String testname=testlist[i];
          dbg("...invoking test" + (i+1) + ": " + testname, dbgFlag);
	  boolean result = tc.executeTest(testname, tc);
	  if (result == true)  {
	    stat.addStatus(testSuiteID + testname, stat.PASS);
	  } else {
	    stat.addStatus(testSuiteID + testname, stat.FAIL);
	  }
          dbg("...finished "+testname+", passed = "+result, dbgFlag);
	}
		
	stat.printSummary(); 

    }
    
}




