package ejb30.issue;

import java.util.*;
import javax.persistence.*;
import java.lang.reflect.*;

import com.sun.ejte.ccl.reporter.SimpleReporterAdapter;

public class TestClient {
	
    EntityManagerFactory emf = 
       Persistence.createEntityManagerFactory("manager", new HashMap());
    EntityManager em = emf.createEntityManager();
    static boolean dbgFlag = false;

    public TestClient() {
    }
      

    public boolean initializeData() {
        
        boolean pass=false;

        DateTime dt1 = example1();
        DateTime dt2 = example2();
        DateTime dt3 = example3();

        try {

	    em.getTransaction().begin();
            em.persist(dt1);
            em.persist(dt2);
            em.persist(dt3);
            em.getTransaction().commit();
            pass = true;
            
        } catch(Throwable e){
            e.printStackTrace();
        }
        
        return pass;

    }

    public boolean testCurrentTime()
    {

        boolean pass=false;
        int exp = 0;
        String jpql = null;
        int updated = 0;
	List results = null;

        // test query setting java.sql.Date field
        try {
	  exp = executeJPQLReturningInt(em, 
		  "SELECT COUNT(dt) FROM DateTime dt");
	  if (dbgFlag) {
            // dbg message
            jpql = "SELECT dt FROM DateTime dt"; 
            System.out.println("-----before update\n"+
		   renderResult(executeJPQLReturningList(em, jpql)));
	    System.out.println("exp = "+ exp);
	  }

          // test bulk update
	  jpql = "UPDATE DateTime dt SET dt.sqlTime = CURRENT_TIME";
	  em.getTransaction().begin();
	  updated = em.createQuery(jpql).executeUpdate();
	  assert exp == updated : "CURRENT_TIME: set time" + 
	    "wrong number of updated instances";
	  em.getTransaction().commit();

	  if (dbgFlag) {            
	    // dbg message
	    System.out.println("updated = "+ updated);
	    jpql = "SELECT dt FROM DateTime dt"; 
            System.out.println("-----after update\n"+
		   renderResult(executeJPQLReturningList(em, jpql)));
	  }
            
	  pass = true;

        } finally {
            if (em.getTransaction().isActive()){
                em.getTransaction().rollback();
            }
	   
        }
        
	return pass;
    }


    public boolean testCurrentTimestamp()
    {

        boolean pass=false;
        int exp = 0;
        String jpql = null;
        int updated = 0;
	List results = null;

        // test query setting java.sql.Timestamp field
        try {

	  exp = executeJPQLReturningInt(em, 
		  "SELECT COUNT(dt) FROM DateTime dt");

	  if (dbgFlag) {
            // dbg message
            jpql = "SELECT dt FROM DateTime dt"; 
            System.out.println("-----before update\n"+
			       renderResult(executeJPQLReturningList(em, jpql)));
	    System.out.println("exp = "+ exp);
	  }

          // test bulk update
	  jpql = "UPDATE DateTime dt SET dt.sqlTimestamp = CURRENT_TIMESTAMP";
	  em.getTransaction().begin();
	  updated = em.createQuery(jpql).executeUpdate();
	  assert exp == updated : "CURRENT_TIMESTAMP -- set timestamp: " + 
	    "wrong number of updated instances";
	  em.getTransaction().commit();
	  System.out.println("updated = "+ updated);
            
	  if (dbgFlag) {
	    // dbg message
	    jpql = "SELECT dt FROM DateTime dt"; 
	    System.out.println("-----after update\n"+
			       renderResult(executeJPQLReturningList(em, jpql)));
	  }

	  pass = true;
        } finally {
            if (em.getTransaction().isActive()){
                em.getTransaction().rollback();
            }
	   
        }
        
	return pass;
    }

    public boolean testCurrentDate()
    {

        boolean pass=false;
        int exp = 0;
        String jpql = null;
        int updated = 0;
	List results = null;

        // test query setting java.sql.Date field
        try {

          // test select
	  exp = executeJPQLReturningInt(em, 
		"SELECT COUNT(dt) FROM DateTime dt WHERE dt.sqlDate >= CURRENT_DATE");
	  if (dbgFlag) {
            // dbg message
            jpql = "SELECT dt FROM DateTime dt"; 
            System.out.println("-----before update\n"+
			       renderResult(executeJPQLReturningList(em, jpql)));
	    System.out.println("exp = "+ exp);
	  }

          // test bulk delete
          jpql = "DELETE FROM DateTime dt WHERE dt.sqlDate >= CURRENT_DATE";
	  em.getTransaction().begin();
	  updated = em.createQuery(jpql).executeUpdate();
	  assert exp == updated : "CURRENT_DATE -- delete date: " + 
	    "wrong number of deleted instances";
	  em.getTransaction().commit();

	  if (dbgFlag) {
            // dbg message
	    System.out.println("updated = "+ updated);
            jpql = "SELECT dt FROM DateTime dt"; 
            System.out.println("-----after update\n"+
			       renderResult(executeJPQLReturningList(em, jpql)));
	  }

	  pass = true;
        } finally {
            if (em.getTransaction().isActive()){
                em.getTransaction().rollback();
            }
	   
        }

	return pass;
    }

    // Helper method executing a JPQL query retuning an int value. 
    private int executeJPQLReturningInt(EntityManager em, String jpql) 
    {
        Query q = em.createQuery(jpql);
        Object result = q.getSingleResult();
        return ((Number)result).intValue();
    }

    // Helper method executing a JPQL query retuning a list. 
    private List executeJPQLReturningList(EntityManager em, String jpql) 
    {
        Query q = em.createQuery(jpql);
	q.setHint("toplink.refresh",true);
        List result = q.getResultList();
        return result;
    }


    private String renderResult(List result) 
    {
      StringBuffer sb = new StringBuffer();
      int s = result.size();
      for(int i=0; i<s; i++){
	DateTime dt = (DateTime) result.get(i);
	sb.append("---dt"+(i+1)+" with id = "+dt.getId()+": ").append("\n");
	sb.append("sqlDate: ").append(dt.getSqlDate()).append("; ");
	sb.append("sqlTime: ").append(dt.getSqlTime()).append("; ");
	sb.append("sqlTimestamp: ").append(dt.getSqlTimestamp()).append("\n");
      }
      return sb.toString();

    }

    public DateTime example1() {
        GregorianCalendar cal = new GregorianCalendar(); 
        cal.set(2001, 6, 1, 3, 45, 32);
        cal.set(Calendar.MILLISECOND, 87);
        
        return buildAttributes(cal);
    }

    public DateTime example2() {
        GregorianCalendar cal = new GregorianCalendar(); 
        cal.set(1981, 11, 31, 23, 59, 59);
        cal.set(Calendar.MILLISECOND, 999);
        return buildAttributes(cal);
    }

    public DateTime example3() {
        Calendar cal = Calendar.getInstance(); 
        cal.set(2030, 0, 1, 23, 45, 32);
        cal.set(Calendar.MILLISECOND, 234);
        return buildAttributes(cal);
    }


    public DateTime buildAttributes(Calendar cal) {
        DateTime dateTime = new DateTime();
        long time = cal.getTime().getTime();;
        dateTime.setSqlDate(new java.sql.Date(time));        
        dateTime.setSqlTime(new java.sql.Time(cal.get(Calendar.HOUR_OF_DAY), 
		 cal.get(Calendar.MINUTE), cal.get(Calendar.SECOND)));        
        dateTime.setSqlTimestamp(new java.sql.Timestamp(time));
        dateTime.setUtilDate(new Date(time));
        dateTime.setCalendar(cal);
        return dateTime;        
    }


    public boolean executeTest(String testname, Object test) {
	Method m=null;
	boolean ret=false;  
	
	try {
	  m = ejb30.issue.TestClient.class.getMethod(testname, new Class[] {});
	  Object[] arg = {};
	  ret = (Boolean) m.invoke(test, (Object[])null);     
	} catch (Exception ex){
	  ex.printStackTrace();
	}

	return ret;

    } 
    

    public void doClose() {
	   
	em.close();
        emf.close();

    }

    public static void dbg(String msg, boolean flag) {
	if (flag) {
            System.out.println(msg);
        }

    }


    public static void main(String[] args) {    

	SimpleReporterAdapter stat = new SimpleReporterAdapter("appserver-sqe");   
	stat.addDescription("EJB3 Issue 1886 Tests");	   
        String testSuiteID = "test";
        String flag = "dbg";

       	String testlist[]={"initializeData",
                           "testCurrentTime",
                           "testCurrentTimestamp",
			   "testCurrentDate"};


        if (args.length == 1) {
           testSuiteID=args[0];
        } 
        if (args.length == 2) {
           testSuiteID=args[0];
           flag=args[1];
           if (flag.equals(new String("true"))) {
             dbgFlag = true;
	   }
        } 

        TestClient tc = new TestClient();

	for (int i=0; i < testlist.length; i++){
	  String testname=testlist[i];
          dbg("...invoking test" + (i+1) + ": " + testname, dbgFlag);
	  boolean result = tc.executeTest(testname, tc);
	  if (result == true)  {
	    stat.addStatus(testSuiteID + testname, stat.PASS);
	  } else {
	    stat.addStatus(testSuiteID + testname, stat.FAIL);
	  }
          dbg("...finished "+testname+", passed = "+result, dbgFlag);
	}
		
	stat.printSummary(); 
	tc.doClose();

    }

   
}
