package ejb30.issue;

import javax.ejb.Stateless;
import javax.ejb.Remote;
import javax.persistence.*;
import java.util.*;


@Stateless
@Remote({ejb30.issue.TestSessionRemote.class})
public class TestSessionBean implements TestSessionRemote {
    
    @PersistenceContext EntityManager em;
    public boolean dbgFlag = true;

    public TestSessionBean() {
    }

    public boolean initializeData() {
        boolean pass= false;

        DateTime dt1 = example1();
        DateTime dt2 = example2();
        DateTime dt3 = example3();

        try {
            em.persist(dt1);
            em.persist(dt2);
            em.persist(dt3);
            pass = true;
            
        } catch(Throwable e){
            e.printStackTrace();
        }
        
        return pass;
    }


    public boolean testCurrentTime() {
        boolean pass= false;

        int exp = 0;
        String jpql = null;
        int updated = 0;
	List results = null;

        // test query setting java.sql.Date field
        try {
	  exp = executeJPQLReturningInt(em, 
		  "SELECT COUNT(dt) FROM DateTime dt");

	  if (dbgFlag) {
	    System.out.println("exp = "+ exp);
            jpql = "SELECT dt FROM DateTime dt"; 
            System.out.println("-----before update\n"+
		   renderResult(executeJPQLReturningList(em, jpql)));
	    System.out.println("exp = "+ exp);
	  }

          // test bulk update
	  jpql = "UPDATE DateTime dt SET dt.sqlTime = CURRENT_TIME";

	  updated = em.createQuery(jpql).executeUpdate();
	  assert exp == updated : "CURRENT_TIME: set time" + 
	    "wrong number of updated instances";

	  if (dbgFlag) {            
	    System.out.println("updated = "+ updated);
	    jpql = "SELECT dt FROM DateTime dt"; 
            System.out.println("-----after update\n"+
		   renderResult(executeJPQLReturningList(em, jpql)));
	  }
            
	  pass = true;

        } catch(Exception ex){

	  ex.printStackTrace();

        }
	return pass;

    }

    public boolean testCurrentTimestamp() {
        boolean pass= false;
        int exp = 0;
        String jpql = null;
        int updated = 0;
	List results = null;

        // test query setting java.sql.Timestamp field
        try {

	  exp = executeJPQLReturningInt(em, 
		  "SELECT COUNT(dt) FROM DateTime dt");

	  if (dbgFlag) {
	    System.out.println("exp = "+ exp);
            jpql = "SELECT dt FROM DateTime dt"; 
            System.out.println("-----before update\n"+
			       renderResult(executeJPQLReturningList(em, jpql)));
	    System.out.println("exp = "+ exp);
	  }

          // test bulk update
	  jpql = "UPDATE DateTime dt SET dt.sqlTimestamp = CURRENT_TIMESTAMP";
	  updated = em.createQuery(jpql).executeUpdate();
	  assert exp == updated : "CURRENT_TIMESTAMP -- set timestamp: " + 
	    "wrong number of updated instances";

            
	  if (dbgFlag) {
	    System.out.println("updated = "+ updated);
	    jpql = "SELECT dt FROM DateTime dt"; 
	    System.out.println("-----after update\n"+
			       renderResult(executeJPQLReturningList(em, jpql)));
	  }

	  pass = true;
        } catch(Exception ex){

	  ex.printStackTrace();

        }
        
	return pass;
    }



    public boolean testCurrentDate() {
        boolean pass= false;
        int exp = 0;
        String jpql = null;
        int updated = 0;
	List results = null;

        // test query setting java.sql.Date field
        try {

          // test select
	  exp = executeJPQLReturningInt(em, 
		"SELECT COUNT(dt) FROM DateTime dt WHERE dt.sqlDate >= CURRENT_DATE");
	  if (dbgFlag) {
	    System.out.println("exp = "+ exp);
            jpql = "SELECT dt FROM DateTime dt"; 
            System.out.println("-----before update\n"+
			       renderResult(executeJPQLReturningList(em, jpql)));
	    System.out.println("exp = "+ exp);
	  }

          // test bulk delete
          jpql = "DELETE FROM DateTime dt WHERE dt.sqlDate >= CURRENT_DATE";
	  updated = em.createQuery(jpql).executeUpdate();
	  assert exp == updated : "CURRENT_DATE -- delete date: " + 
	    "wrong number of deleted instances";

	  if (dbgFlag) {
	    System.out.println("updated = "+ updated);
            jpql = "SELECT dt FROM DateTime dt"; 
            System.out.println("-----after update\n"+
			       renderResult(executeJPQLReturningList(em, jpql)));
	  }

	  pass=true;

        } catch(Exception ex){

	  ex.printStackTrace();

        }
        
        return pass;
    }

    // Helper method executing a JPQL query retuning an int value. 
    private int executeJPQLReturningInt(EntityManager em, String jpql) 
    {
        Query q = em.createQuery(jpql);
        Object result = q.getSingleResult();
        return ((Number)result).intValue();
    }

    // Helper method executing a JPQL query retuning a list. 
    private List executeJPQLReturningList(EntityManager em, String jpql) 
    {
        Query q = em.createQuery(jpql);
	q.setHint("toplink.refresh",true);
        List result = q.getResultList();
        return result;
    }


    private String renderResult(List result) 
    {
      StringBuffer sb = new StringBuffer();
      int s = result.size();
      for(int i=0; i<s; i++){
	DateTime dt = (DateTime) result.get(i);
	sb.append("---dt"+(i+1)+" with id = "+dt.getId()+": ").append("\n");
	sb.append("sqlDate: ").append(dt.getSqlDate()).append("; ");
	sb.append("sqlTime: ").append(dt.getSqlTime()).append("; ");
	sb.append("sqlTimestamp: ").append(dt.getSqlTimestamp()).append("\n");
      }
      return sb.toString();

    }

    public DateTime example1() {
        GregorianCalendar cal = new GregorianCalendar(); 
        cal.set(2001, 6, 1, 3, 45, 32);
        cal.set(Calendar.MILLISECOND, 87);
        
        return buildAttributes(cal);
    }

    public DateTime example2() {
        GregorianCalendar cal = new GregorianCalendar(); 
        cal.set(1981, 11, 31, 23, 59, 59);
        cal.set(Calendar.MILLISECOND, 999);
        return buildAttributes(cal);
    }

    public DateTime example3() {
        Calendar cal = Calendar.getInstance(); 
        cal.set(2030, 0, 1, 23, 45, 32);
        cal.set(Calendar.MILLISECOND, 234);
        return buildAttributes(cal);
    }


    public DateTime buildAttributes(Calendar cal) {
        DateTime dateTime = new DateTime();
        long time = cal.getTime().getTime();;
        dateTime.setSqlDate(new java.sql.Date(time));        
        dateTime.setSqlTime(new java.sql.Time(cal.get(Calendar.HOUR_OF_DAY), 
		 cal.get(Calendar.MINUTE), cal.get(Calendar.SECOND)));        
        dateTime.setSqlTimestamp(new java.sql.Timestamp(time));
        dateTime.setUtilDate(new Date(time));
        dateTime.setCalendar(cal);
        return dateTime;        
    }

    private void doFlush() throws PersistenceException  {
	System.out.println("Entering doFlush method");
	try {
		em.flush();
	} catch (PersistenceException pe) {
	 	throw new PersistenceException("Unexpected Exception caught while flushing: " + pe);
        }
    }
    
}




